@@ -44,7 +44,13 @@ class LatentBuildSlaveFailedToSubstantiate(Exception):
     pass
 
 
-class IChangeSource(Interface):
+class IPlugin(Interface):
+    """
+    Base interface for all Buildbot plugins
+    """
+
+
+class IChangeSource(IPlugin):
 
     """
     Service which feeds Change objects to the changemaster. When files or
@@ -151,7 +157,7 @@ class IStatus(Interface):
         """Return a list of ISchedulerStatus objects for all
         currently-registered Schedulers."""
 
-    def getBuilderNames(categories=None):
+    def getBuilderNames(categories=None, tags=None):
         """Return a list of the names of all current Builders."""
     def getBuilder(name):
         """Return the IBuilderStatus object for a given named Builder. Raises
@@ -328,6 +334,9 @@ class IBuilderStatus(Interface):
     def getName():
         """Return the name of this Builder (a string)."""
 
+    def getTags():
+        """Return the tags of this builder (a list of strings)"""
+
     def getCategory():
         """Return the category of this builder (a string)."""
 
@@ -883,7 +892,7 @@ class IStatusLogConsumer(Interface):
         """The log has finished sending chunks to the consumer."""
 
 
-class IStatusReceiver(Interface):
+class IStatusReceiver(IPlugin):
 
     """I am an object which can receive build status updates. I may be
     subscribed to an IStatus, an IBuilderStatus, or an IBuildStatus."""
@@ -1141,7 +1150,7 @@ class ILogObserver(Interface):
         pass
 
 
-class IBuildSlave(Interface):
+class IBuildSlave(IPlugin):
     # this is a marker interface for the BuildSlave class
     pass
 
@@ -1266,7 +1275,7 @@ class IProperties(Interface):
         """
 
 
-class IScheduler(Interface):
+class IScheduler(IPlugin):
     pass
 
 
@@ -1285,3 +1294,10 @@ class IBuildStepFactory(Interface):
 
     def buildStep():
         pass
+
+
+class IBuildStep(IPlugin):
+    """
+    A build step
+    """
+    # Currently has nothing
