@@ -13,19 +13,23 @@
 #
 # Copyright Buildbot Team Members
 
-import time
 import operator
 import re
+import time
 import urllib
-from twisted.internet import defer
+
 from buildbot import util
+from buildbot.changes import changes
 from buildbot.status import builder
 from buildbot.status.web.base import HtmlResource
-from buildbot.changes import changes
+from twisted.internet import defer
 
-class DoesNotPassFilter(Exception): pass # Used for filtering revs
 
-def getResultsClass(results, prevResults, inProgress):
+class DoesNotPassFilter(Exception):
+    pass  # Used for filtering revs
+
+
+def getResultsClass(results, prevResults, inProgress, inBuilder):
     """Given the current and past results, return the class that will be used
     by the css to display the right color for a box."""
 
@@ -33,7 +37,10 @@ def getResultsClass(results, prevResults, inProgress):
         return "running"
 
     if results is None:
-        return "notstarted"
+        if inBuilder:
+            return "notstarted"
+        else:
+            return "notinbuilder"
 
     if results == builder.SUCCESS:
         return "success"
@@ -52,17 +59,22 @@ def getResultsClass(results, prevResults, inProgress):
             return "failure"
         else:
             # The previous build also failed.
-            return "warnings"
-  
+            return "failure-again"
+
     # Any other results? Like EXCEPTION?
     return "exception"
 
-class ANYBRANCH: pass # a flag value, used below
+
+class ANYBRANCH:
+    pass  # a flag value, used below
+
 
 class DevRevision:
+
     """Helper class that contains all the information we need for a revision."""
 
     def __init__(self, change):
+        self.codebase = change.codebase
         self.revision = change.revision
         self.comments = change.comments
         self.who = change.who
@@ -74,21 +86,22 @@ class DevRevision:
 
 
 class DevBuild:
+
     """Helper class that contains all the information we need for a build."""
 
-    def __init__(self, revision, build, details):
-        self.revision = revision
-        self.results =  build.getResults()
+    def __init__(self, build, details):
+        self.results = build.getResults()
         self.number = build.getNumber()
         self.isFinished = build.isFinished()
         self.text = build.getText()
         self.eta = build.getETA()
         self.details = details
         self.when = build.getTimes()[0]
-        self.source = build.getSourceStamp()
+        self.sourceStamps = build.getSourceStamps()
 
 
 class ConsoleStatusResource(HtmlResource):
+
     """Main console class. It displays a user-oriented status page.
     Every change is a line in the page, and it shows the result of the first
     build with this change for each slave."""
@@ -114,9 +127,9 @@ class ConsoleStatusResource(HtmlResource):
     def getChangeManager(self, request):
         return request.site.buildbot_service.parent.change_svc
 
-    ##
-    ## Data gathering functions
-    ##
+    #
+    # Data gathering functions
+    #
 
     def getHeadBuild(self, builder):
         """Get the most recent build for the given builder.
@@ -133,51 +146,45 @@ class ConsoleStatusResource(HtmlResource):
     def fetchChangesFromHistory(self, status, max_depth, max_builds, debugInfo):
         """Look at the history of the builders and try to fetch as many changes
         as possible. We need this when the main source does not contain enough
-        sourcestamps. 
+        sourcestamps.
 
         max_depth defines how many builds we will parse for a given builder.
         max_builds defines how many builds total we want to parse. This is to
             limit the amount of time we spend in this function.
-        
+
         This function is sub-optimal, but the information returned by this
         function is cached, so this function won't be called more than once.
         """
-        
+
         allChanges = list()
         build_count = 0
         for builderName in status.getBuilderNames()[:]:
             if build_count > max_builds:
                 break
-            
+
             builder = status.getBuilder(builderName)
             build = self.getHeadBuild(builder)
             depth = 0
             while build and depth < max_depth and build_count < max_builds:
                 depth += 1
                 build_count += 1
-                sourcestamp = build.getSourceStamp()
+                sourcestamp = build.getSourceStamps()[0]
                 allChanges.extend(sourcestamp.changes[:])
                 build = build.getPreviousBuild()
 
         debugInfo["source_fetch_len"] = len(allChanges)
-        return allChanges                
+        return allChanges
 
-    @defer.deferredGenerator
-    def getAllChanges(self, request, status, debugInfo):
+    @defer.inlineCallbacks
+    def getAllChanges(self, request, status, numChanges, debugInfo):
         master = request.site.buildbot_service.master
 
-        wfd = defer.waitForDeferred(
-                master.db.changes.getRecentChanges(25))
-        yield wfd
-        chdicts = wfd.getResult()
+        chdicts = yield master.db.changes.getRecentChanges(numChanges)
 
         # convert those to Change instances
-        wfd = defer.waitForDeferred(
-            defer.gatherResults([
-                changes.Change.fromChdict(master, chdict)
-                for chdict in chdicts ]))
-        yield wfd
-        allChanges = wfd.getResult()
+        allChanges = yield defer.gatherResults([
+            changes.Change.fromChdict(master, chdict)
+            for chdict in chdicts])
 
         allChanges.sort(key=self.comparator.getSortingKey())
 
@@ -191,14 +198,14 @@ class ConsoleStatusResource(HtmlResource):
             prevChange = change
         allChanges = newChanges
 
-        yield allChanges
+        defer.returnValue(allChanges)
 
     def getBuildDetails(self, request, builderName, build):
         """Returns an HTML list of failures for a given build."""
         details = {}
         if not build.getLogs():
             return details
-        
+
         for step in build.getSteps():
             (result, reason) = step.getResults()
             if result == builder.FAILURE:
@@ -207,7 +214,7 @@ class ConsoleStatusResource(HtmlResource):
                 # Remove html tags from the error text.
                 stripHtml = re.compile(r'<.*?>')
                 strippedDetails = stripHtml.sub('', ' '.join(step.getText()))
-                
+
                 details['buildername'] = builderName
                 details['status'] = strippedDetails
                 details['reason'] = reason
@@ -217,7 +224,7 @@ class ConsoleStatusResource(HtmlResource):
                     for log in step.getLogs():
                         logname = log.getName()
                         logurl = request.childLink(
-                          "../builders/%s/builds/%s/steps/%s/logs/%s" % 
+                            "../builders/%s/builds/%s/steps/%s/logs/%s" %
                             (urllib.quote(builderName),
                              build.getNumber(),
                              urllib.quote(name),
@@ -225,71 +232,41 @@ class ConsoleStatusResource(HtmlResource):
                         logs.append(dict(url=logurl, name=logname))
         return details
 
-    def getBuildsForRevision(self, request, builder, builderName, lastRevision,
-                             numBuilds, debugInfo):
+    def getBuildsForRevision(self, request, builder, builderName, codebase,
+                             lastRevision, numBuilds, debugInfo):
         """Return the list of all the builds for a given builder that we will
         need to be able to display the console page. We start by the most recent
         build, and we go down until we find a build that was built prior to the
         last change we are interested in."""
 
-        revision = lastRevision 
-
         builds = []
         build = self.getHeadBuild(builder)
         number = 0
         while build and number < numBuilds:
             debugInfo["builds_scanned"] += 1
-            number += 1
 
-            # Get the last revision in this build.
-            # We first try "got_revision", but if it does not work, then
-            # we try "revision".
-            got_rev = build.getProperty("got_revision", build.getProperty("revision", -1))
-            if got_rev != -1 and not self.comparator.isValidRevision(got_rev):
-                got_rev = -1
-
-            # We ignore all builds that don't have last revisions.
-            # TODO(nsylvain): If the build is over, maybe it was a problem
-            # with the update source step. We need to find a way to tell the
-            # user that his change might have broken the source update.
-            if got_rev != -1:
+            # With multiple codebases cannot determine the last required build,
+            # so take them all except forced builds.
+            if len(build.getChanges()):
                 details = self.getBuildDetails(request, builderName, build)
-                devBuild = DevBuild(got_rev, build, details)
+                devBuild = DevBuild(build, details)
                 builds.append(devBuild)
-
-                # Now break if we have enough builds.
-                current_revision = self.getChangeForBuild(
-                    build, revision)
-                if self.comparator.isRevisionEarlier(
-                    devBuild, current_revision):
-                    break
+                number += 1
 
             build = build.getPreviousBuild()
 
         return builds
 
-    def getChangeForBuild(self, build, revision):
-        if not build or not build.getChanges(): # Forced build
-            return DevBuild(revision, build, None)
-        
-        for change in build.getChanges():
-            if change.revision == revision:
-                return change
-
-        # No matching change, return the last change in build.
-        changes = list(build.getChanges())
-        changes.sort(key=self.comparator.getSortingKey())
-        return changes[-1]
-    
-    def getAllBuildsForRevision(self, status, request, lastRevision, numBuilds,
-                                categories, builders, debugInfo):
+    def getAllBuildsForRevision(self, status, request, codebase, lastRevision,
+                                numBuilds, tags, builders, debugInfo):
         """Returns a dictionary of builds we need to inspect to be able to
         display the console page. The key is the builder name, and the value is
         an array of build we care about. We also returns a dictionary of
         builders we care about. The key is it's category.
- 
+
+        codebase is the codebase to get revisions from
         lastRevision is the last revision we want to display in the page.
-        categories is a list of categories to display. It is coming from the
+        tags is a list of tags to display. It is coming from the
             HTTP GET parameters.
         builders is a list of builders to display. It is coming from the HTTP
             GET parameters.
@@ -307,62 +284,53 @@ class ConsoleStatusResource(HtmlResource):
             builder = status.getBuilder(builderName)
 
             # Make sure we are interested in this builder.
-            if categories and builder.category not in categories:
+            if tags and not builder.matchesAnyTag(tags):
                 continue
             if builders and builderName not in builders:
                 continue
 
             # We want to display this builder.
-            category = builder.category or "default"
-            # Strip the category to keep only the text before the first |.
-            # This is a hack to support the chromium usecase where they have
-            # multiple categories for each slave. We use only the first one.
-            # TODO(nsylvain): Create another way to specify "display category"
-            #     in master.cfg.
-            category = category.split('|')[0]
-            if not builderList.get(category):
-                builderList[category] = []
+            builder_tags = builder.getTags() or ["default"]
+            for tag in builder_tags:
+                # Append this builder to the dictionary of builders.
+                builderList.setdefault(tag, []).append(builderName)
 
-            # Append this builder to the dictionary of builders.
-            builderList[category].append(builderName)
             # Set the list of builds for this builder.
             allBuilds[builderName] = self.getBuildsForRevision(request,
                                                                builder,
                                                                builderName,
+                                                               codebase,
                                                                lastRevision,
                                                                numBuilds,
                                                                debugInfo)
 
         return (builderList, allBuilds)
 
-
-    ##
-    ## Display functions
-    ##
-
-    def displayCategories(self, builderList, debugInfo):
-        """Display the top category line."""
+    #
+    # Display functions
+    #
+    def displayTags(self, builderList, debugInfo):
+        """Display the top tags line."""
 
         count = 0
-        for category in builderList:
-            count += len(builderList[category])
+        for tag in builderList:
+            count += len(builderList[tag])
+
+        tags = sorted(builderList.keys())
 
-        categories = builderList.keys()
-        categories.sort()
-        
         cs = []
-        
-        for category in categories:            
+
+        for tag in tags:
             c = {}
 
-            c["name"] = category
+            c["name"] = tag
 
             # To be able to align the table correctly, we need to know
-            # what percentage of space this category will be taking. This is
-            # (#Builders in Category) / (#Builders Total) * 100.
-            c["size"] = (len(builderList[category]) * 100) / count            
+            # what percentage of space this tag will be taking. This is
+            # (#Builders in tag) / (#Builders Total) * 100.
+            c["size"] = (len(builderList[tag]) * 100) / count
             cs.append(c)
-            
+
         return cs
 
     def displaySlaveLine(self, status, builderList, debugInfo):
@@ -372,44 +340,56 @@ class ConsoleStatusResource(HtmlResource):
         nbSlaves = 0
 
         # Get the number of builders.
-        for category in builderList:
-            nbSlaves += len(builderList[category])
+        for tag in builderList:
+            nbSlaves += len(builderList[tag])
 
-        # Get the categories, and order them alphabetically.
-        categories = builderList.keys()
-        categories.sort()
+        # Get the tags, and order them alphabetically.
+        tags = sorted(builderList.keys())
 
         slaves = {}
 
-        # For each category, we display each builder.
-        for category in categories:
-            slaves[category] = []
-            # For each builder in this category, we set the build info and we
+        # For each tag, we display each builder.
+        for tag in tags:
+            slaves[tag] = []
+            # For each builder in this tag, we set the build info and we
             # display the box.
-            for builder in builderList[category]:
+            for bldr in builderList[tag]:
                 s = {}
                 s["color"] = "notstarted"
-                s["pageTitle"] = builder
-                s["url"] = "./builders/%s" % urllib.quote(builder)
-                state, builds = status.getBuilder(builder).getState()
+                s["pageTitle"] = bldr
+                s["url"] = "./builders/%s" % urllib.quote(bldr)
+                state, builds = status.getBuilder(bldr).getState()
                 # Check if it's offline, if so, the box is purple.
                 if state == "offline":
                     s["color"] = "offline"
                 else:
                     # If not offline, then display the result of the last
                     # finished build.
-                    build = self.getHeadBuild(status.getBuilder(builder))
+                    build = self.getHeadBuild(status.getBuilder(bldr))
                     while build and not build.isFinished():
                         build = build.getPreviousBuild()
 
                     if build:
                         s["color"] = getResultsClass(build.getResults(), None,
-                                                      False)
+                                                     False, True)
 
-                slaves[category].append(s)
+                slaves[tag].append(s)
 
         return slaves
 
+    def isCodebaseInBuild(self, build, codebase):
+        """Check if codebase is used in build"""
+        return any(ss.codebase == codebase for ss in build.sourceStamps)
+
+    def isRevisionInBuild(self, build, revision):
+        """ Check if revision is in changes in build """
+        for ss in build.sourceStamps:
+            if ss.codebase == revision.codebase:
+                for change in ss.changes:
+                    if change.revision == revision.revision:
+                        return True
+        return False
+
     def displayStatusLine(self, builderList, allBuilds, revision, debugInfo):
         """Display the boxes that represent the status of each builder in the
         first build "revision" was in. Returns an HTML list of errors that
@@ -417,33 +397,36 @@ class ConsoleStatusResource(HtmlResource):
 
         details = []
         nbSlaves = 0
-        for category in builderList:
-            nbSlaves += len(builderList[category])
+        for tag in builderList:
+            nbSlaves += len(builderList[tag])
+
+        # Sort the tags.
+        tags = sorted(builderList.keys())
 
-        # Sort the categories.
-        categories = builderList.keys()
-        categories.sort()
-        
         builds = {}
-  
-        # Display the boxes by category group.
-        for category in categories:
-  
-            builds[category] = []
-            
-            # Display the boxes for each builder in this category.
-            for builder in builderList[category]:
+
+        # Display the boxes by tag group.
+        for tag in tags:
+
+            builds[tag] = []
+
+            # Display the boxes for each builder in this tag.
+            for bldr in builderList[tag]:
                 introducedIn = None
                 firstNotIn = None
+                # If there is no builds default to True
+                inBuilder = len(allBuilds[bldr]) == 0
 
-                # Find the first build that does not include the revision.
-                for build in allBuilds[builder]:
-                    if self.comparator.isRevisionEarlier(build, revision):
+                # Find the first build that include the revision.
+                for build in allBuilds[bldr]:
+                    if introducedIn:
                         firstNotIn = build
                         break
-                    else:
-                        introducedIn = build
-                        
+                    elif self.isCodebaseInBuild(build, revision.codebase):
+                        inBuilder = True
+                        if self.isRevisionInBuild(build, revision):
+                            introducedIn = build
+
                 # Get the results of the first build with the revision, and the
                 # first build that does not include the revision.
                 results = None
@@ -458,34 +441,34 @@ class ConsoleStatusResource(HtmlResource):
                     isRunning = True
 
                 url = "./waterfall"
-                pageTitle = builder
-                tag = ""
+                pageTitle = bldr
+                nice_tag = ""
                 current_details = {}
                 if introducedIn:
                     current_details = introducedIn.details or ""
-                    url = "./buildstatus?builder=%s&number=%s" % (urllib.quote(builder),
-                                                                  introducedIn.number)
+                    url = "./buildstatus?builder=%s&amp;number=%s" % (urllib.quote(bldr),
+                                                                      introducedIn.number)
                     pageTitle += " "
                     pageTitle += urllib.quote(' '.join(introducedIn.text), ' \n\\/:')
 
-                    builderStrip = builder.replace(' ', '')
+                    builderStrip = bldr.replace(' ', '')
                     builderStrip = builderStrip.replace('(', '')
                     builderStrip = builderStrip.replace(')', '')
                     builderStrip = builderStrip.replace('.', '')
-                    tag = "Tag%s%s" % (builderStrip, introducedIn.number)
+                    nice_tag = "Tag%s%s" % (builderStrip, introducedIn.number)
 
                 if isRunning:
                     pageTitle += ' ETA: %ds' % (introducedIn.eta or 0)
-                    
-                resultsClass = getResultsClass(results, previousResults, isRunning)
 
-                b = {}                
+                resultsClass = getResultsClass(results, previousResults, isRunning, inBuilder)
+
+                b = {}
                 b["url"] = url
                 b["pageTitle"] = pageTitle
                 b["color"] = resultsClass
-                b["tag"] = tag
+                b["tag"] = nice_tag
 
-                builds[category].append(b)
+                builds[tag].append(b)
 
                 # If the box is red, we add the explaination in the details
                 # section.
@@ -503,7 +486,7 @@ class ConsoleStatusResource(HtmlResource):
                 for rev in reversed(revisions):
                     yield DevRevision(rev)
             else:
-                for index,rev in enumerate(reversed(revisions)):
+                for index, rev in enumerate(reversed(revisions)):
                     if index >= max_revs:
                         break
                     yield DevRevision(rev)
@@ -512,7 +495,7 @@ class ConsoleStatusResource(HtmlResource):
                 if max_revs and index >= max_revs:
                     break
                 try:
-                    for field,acceptable in filter.iteritems():
+                    for field, acceptable in filter.iteritems():
                         if not hasattr(rev, field):
                             raise DoesNotPassFilter
                         if type(acceptable) in (str, unicode):
@@ -525,36 +508,38 @@ class ConsoleStatusResource(HtmlResource):
                 except DoesNotPassFilter:
                     pass
 
-    def displayPage(self, request, status, builderList, allBuilds, revisions,
-                    categories, repository, project, branch, debugInfo):
+    def displayPage(self, request, status, builderList, allBuilds, codebase,
+                    revisions, tags, repository, project, branch,
+                    debugInfo):
         """Display the console page."""
         # Build the main template directory with all the informations we have.
         subs = dict()
         subs["branch"] = branch or 'trunk'
         subs["repository"] = repository
         subs["project"] = project
-        if categories:
-            subs["categories"] = ' '.join(categories)
+        subs["codebase"] = codebase
+        if tags:
+            subs["tags"] = ' '.join(tags)
         subs["time"] = time.strftime("%a %d %b %Y %H:%M:%S",
                                      time.localtime(util.now()))
         subs["debugInfo"] = debugInfo
         subs["ANYBRANCH"] = ANYBRANCH
 
         if builderList:
-            subs["categories"] = self.displayCategories(builderList, debugInfo)
+            subs["tags"] = self.displayTags(builderList, debugInfo)
             subs['slaves'] = self.displaySlaveLine(status, builderList, debugInfo)
         else:
-            subs["categories"] = []
+            subs["tags"] = []
 
         subs['revisions'] = []
 
         # For each revision we show one line
         for revision in revisions:
             r = {}
-            
+
             # Fill the dictionary with this new information
             r['id'] = revision.revision
-            r['link'] = revision.revlink 
+            r['link'] = revision.revlink
             r['who'] = revision.who
             r['date'] = revision.date
             r['comments'] = revision.comments
@@ -563,14 +548,14 @@ class ConsoleStatusResource(HtmlResource):
 
             # Display the status for all builders.
             (builds, details) = self.displayStatusLine(builderList,
-                                            allBuilds,
-                                            revision,
-                                            debugInfo)
+                                                       allBuilds,
+                                                       revision,
+                                                       debugInfo)
             r['builds'] = builds
             r['details'] = details
 
             # Calculate the td span for the comment and the details.
-            r["span"] = len(builderList) + 2            
+            r["span"] = len(builderList) + 2
 
             subs['revisions'].append(r)
 
@@ -580,7 +565,6 @@ class ConsoleStatusResource(HtmlResource):
         debugInfo["load_time"] = time.time() - debugInfo["load_time"]
         return subs
 
-
     def content(self, request, cxt):
         "This method builds the main console view display."
 
@@ -601,7 +585,7 @@ class ConsoleStatusResource(HtmlResource):
         if not reload_time:
             reload_time = 60
 
-        # Append the tag to refresh the page. 
+        # Append the tag to refresh the page.
         if reload_time is not None and reload_time != 0:
             cxt['refresh'] = reload_time
 
@@ -610,8 +594,10 @@ class ConsoleStatusResource(HtmlResource):
         debugInfo["load_time"] = time.time()
 
         # get url parameters
-        # Categories to show information for.
-        categories = request.args.get("category", [])
+        # tags to show information for.
+        tags = request.args.get("tag", [])
+        if not tags:
+            tags = request.args.get("category", [])
         # List of all builders to show on the page.
         builders = request.args.get("builder", [])
         # Repo used to filter the changes shown.
@@ -620,6 +606,8 @@ class ConsoleStatusResource(HtmlResource):
         project = request.args.get("project", [None])[0]
         # Branch used to filter the changes shown.
         branch = request.args.get("branch", [ANYBRANCH])[0]
+        # Codebase used to filter the changes shown.
+        codebase = request.args.get("codebase", [None])[0]
         # List of all the committers name to display on the page.
         devName = request.args.get("name", [])
 
@@ -628,16 +616,12 @@ class ConsoleStatusResource(HtmlResource):
 
         # Keep only the revisions we care about.
         # By default we process the last 40 revisions.
-        # If a dev name is passed, we look for the changes by this person in the
-        # last 80 revisions.
         numRevs = int(request.args.get("revs", [40])[0])
-        if devName:
-            numRevs *= 2
-        numBuilds = numRevs
 
         # Get all changes we can find.  This is a DB operation, so it must use
         # a deferred.
-        d = self.getAllChanges(request, status, debugInfo)
+        d = self.getAllChanges(request, status, numRevs, debugInfo)
+
         def got_changes(allChanges):
             debugInfo["source_all"] = len(allChanges)
 
@@ -650,8 +634,10 @@ class ConsoleStatusResource(HtmlResource):
                 revFilter['repository'] = repository
             if project:
                 revFilter['project'] = project
+            if codebase is not None:
+                revFilter['codebase'] = codebase
             revisions = list(self.filterRevisions(allChanges, max_revs=numRevs,
-                                                            filter=revFilter))
+                                                  filter=revFilter))
             debugInfo["revision_final"] = len(revisions)
 
             # Fetch all the builds for all builders until we get the next build
@@ -663,18 +649,20 @@ class ConsoleStatusResource(HtmlResource):
                 debugInfo["last_revision"] = lastRevision
 
                 (builderList, allBuilds) = self.getAllBuildsForRevision(status,
-                                                    request,
-                                                    lastRevision,
-                                                    numBuilds,
-                                                    categories,
-                                                    builders,
-                                                    debugInfo)
+                                                                        request,
+                                                                        codebase,
+                                                                        lastRevision,
+                                                                        numRevs,
+                                                                        tags,
+                                                                        builders,
+                                                                        debugInfo)
 
             debugInfo["added_blocks"] = 0
 
             cxt.update(self.displayPage(request, status, builderList,
-                                        allBuilds, revisions, categories,
-                                        repository, project, branch, debugInfo))
+                                        allBuilds, codebase, revisions,
+                                        tags, repository, project,
+                                        branch, debugInfo))
 
             templates = request.site.buildbot_service.templates
             template = templates.get_template("console.html")
@@ -683,14 +671,16 @@ class ConsoleStatusResource(HtmlResource):
         d.addCallback(got_changes)
         return d
 
+
 class RevisionComparator(object):
+
     """Used for comparing between revisions, as some
     VCS use a plain counter for revisions (like SVN)
     while others use different concepts (see Git).
     """
-    
+
     # TODO (avivby): Should this be a zope interface?
-    
+
     def isRevisionEarlier(self, first_change, second_change):
         """Used for comparing 2 changes"""
         raise NotImplementedError
@@ -701,18 +691,22 @@ class RevisionComparator(object):
 
     def getSortingKey(self):
         raise NotImplementedError
-    
+
+
 class TimeRevisionComparator(RevisionComparator):
+
     def isRevisionEarlier(self, first, second):
         return first.when < second.when
 
     def isValidRevision(self, revision):
-        return True # No general way of determining
+        return True  # No general way of determining
 
     def getSortingKey(self):
         return operator.attrgetter('when')
 
+
 class IntegerRevisionComparator(RevisionComparator):
+
     def isRevisionEarlier(self, first, second):
         try:
             return int(first.revision) < int(second.revision)
@@ -728,4 +722,3 @@ class IntegerRevisionComparator(RevisionComparator):
 
     def getSortingKey(self):
         return operator.attrgetter('revision')
-
